#include "sensorlistwidget.h"

#include <QDebug>
#include <QHeaderView>
#include <QScroller>

SensorListWidget::SensorListWidget(const bool showHidden, QWidget *parent): QTableWidget(parent),
	showHidden_(showHidden)
{
	setColumnCount(3);
	setSelectionBehavior(QAbstractItemView::SelectRows);
	horizontalHeader()->setSectionResizeMode(0, QHeaderView::Stretch);
	QScroller::grabGesture(this, QScroller::LeftMouseButtonGesture);
	setAutoScroll(true);
	setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
	sensorsChanged(std::vector<Sensor>());
	verticalHeader()->hide();
}

SensorListWidget::SensorListWidget(SensorStore& sensorStore, const bool showHidden,
                                   QWidget* parent): QTableWidget (parent), showHidden_(showHidden)
{
	sensorsChanged(*(sensorStore.getSensors()));
}

void SensorListWidget::sensorsChanged(std::vector<Sensor> sensors)
{
	clear();
	setHorizontalHeaderItem(0, new QTableWidgetItem("Sensor"));
	setHorizontalHeaderItem(1, new QTableWidgetItem("Value"));
	setHorizontalHeaderItem(2, new QTableWidgetItem("Time"));
	size_t listLen = 0;
	for(size_t i = 0; i < sensors.size(); ++i)
		if(showHidden_ || !sensors[i].hidden)
			++listLen;
	setRowCount(static_cast<int>(listLen));
	size_t row = 0;
	for(size_t i = 0; i < sensors.size(); ++i)
	{
		if(showHidden_ || !sensors[i].hidden)
		{
			QString itemString;
			itemString.append(QString::number(sensors[i].field));
			itemString.append(' ');

			if(sensors[i].type == Sensor::TYPE_DOOR)
			{
				if(static_cast<bool>(sensors[i].field))
					itemString.append("\"Open\"");
				else itemString.append("\"Closed\"");
			}
			else if(sensors[i].type == Sensor::TYPE_AUDIO_OUTPUT)
			{
				if(static_cast<bool>(sensors[i].field))
					itemString.append("\"Playing\"");
				else itemString.append("\"Silent\"");
			}

			setItem(static_cast<int>(row), 0, new QTableWidgetItem(sensors[i].name + (sensors[i].hidden ? " (H)" : "")));
			setItem(static_cast<int>(row), 1, new QTableWidgetItem(itemString));
			if(sensors[i].type <= 128)
				setItem(static_cast<int>(row), 2, new QTableWidgetItem(sensors[i].lastSeen.time().toString("hh:mm")));
			++row;
		}
	}
	sortItems(0, Qt::AscendingOrder);
	resizeColumnsToContents();
}

void SensorListWidget::setShowHidden(const bool showHidden)
{
	showHidden_=showHidden;
}

